// This file is part of the Acts project.
//
// Copyright (C) 2021 CERN for the benefit of the Acts project
//
// This Source Code Form is subject to the terms of the Mozilla Public
// License, v. 2.0. If a copy of the MPL was not distributed with this
// file, You can obtain one at http://mozilla.org/MPL/2.0/.

#include "Acts/Plugins/Json/AlgebraJsonConverter.hpp"

void Acts::to_json(nlohmann::json& j, const Acts::Transform3& r) {
  auto translation = r.translation();
  if (translation != Acts::Vector3(0., 0., 0)) {
    std::array<Acts::ActsScalar, 3> tdata = {translation.x(), translation.y(),
                                             translation.z()};
    j["translation"] = tdata;
  } else {
    j["translation"] = nlohmann::json();
  }

  auto rotation = r.rotation();
  if (rotation != Acts::RotationMatrix3::Identity()) {
    std::array<Acts::ActsScalar, 9> rdata = {
        rotation(0, 0), rotation(0, 1), rotation(0, 2),
        rotation(1, 0), rotation(1, 1), rotation(1, 2),
        rotation(2, 0), rotation(2, 1), rotation(2, 2)};
    j["rotation"] = rdata;
  } else {
    j["rotation"] = nlohmann::json();
  }
}

void Acts::from_json(const nlohmann::json& j, Acts::Transform3& t) {
  t = Acts::Transform3::Identity();
  if (j.find("translation") != j.end() and not j["translation"].empty()) {
    std::array<Acts::ActsScalar, 3> tdata = j["translation"];
    t.pretranslate(Acts::Vector3(tdata[0], tdata[1], tdata[2]));
  }
  if (j.find("rotation") != j.end() and not j["rotation"].empty()) {
    std::array<Acts::ActsScalar, 9> rdata = j["rotation"];
    Acts::RotationMatrix3 rot;
    rot << rdata[0], rdata[1], rdata[2], rdata[3], rdata[4], rdata[5], rdata[6],
        rdata[7], rdata[8];
    t.prerotate(rot);
  }
}
